'use strict';

const coDwfCore = require('@sap/dwf-core');
const coProtocol = new coDwfCore.protocol();

module.exports.getDwf = function(isQuery) {
  return coProtocol.getDwf(isQuery);
};

module.exports.setFreeText = function(ioDwf, iValue) {
  let lsValue = '' + iValue;
  ioDwf.protocol.clientQuery.txt = lsValue;
  return;
};

module.exports.getFreeText = function(ioDwf) {
  let lsValue = ioDwf.protocol.clientQuery.txt;
  return lsValue;
};

module.exports.resetFilter = function(ioDwf) {
  ioDwf.protocol.clientQuery.flt = {};
};

module.exports.addFilter = function(ioDwf, isKey, iValue) {
  let lsValue = '' + iValue;
  let loFlt = ioDwf.protocol.clientQuery.flt;
  if ( ! loFlt) {
    ioDwf.protocol.clientQuery.flt = {};
  }
  loFlt[isKey] = lsValue;
  return;
};

module.exports.getFilter = function(ioDwf, isKey) {
  let loFlt = ioDwf.protocol.clientQuery.flt;
  let lsValue;
  if (loFlt) {
    lsValue = loFlt[isKey];
  }
  return lsValue;
};

/**
 * Gets the default callback function that can be used to push the result to the http response
 *
 * @ioReq object the http request
 * @ioRes object the http response
 *
 * @returns function(ioError, ioResult)
 */
module.exports.getCallback = function(ioReq, ioRes) {
  let lfCallback = function(ioError, ioResult) {
    if (ioError) {
      ioRes.status(500).send(ioReq.dwf.protocol.getErrorResponse(ioError));
    } else {
      ioRes.send(ioReq.dwf.protocol.getResultResponse(ioResult));
    }
  };
  return lfCallback;
};

/**
 * Queries the DB, by exposing an alternative way of using the protocol
 *
 * @param ifCb function(ioError, ioResult)
 * @param ioDb Client as defined in the node-hdb node.js module
 * @param ioDwf
 * @param isSql
 * @param iasParam
 * @param iasFreeTextColumn optional
 * @param isResultKey optional
 *
 */
module.exports.query = function(ifCb, ioDb, ioDwf, isSql, iasParam, iasFreeTextColumn, isResultKey) {
  ioDwf.protocol.executeQuery({
    'request': {
      'db': ioDb,
      'dwf': ioDwf
    },
    'dataSource': isSql,
    'freeTextColumns': iasFreeTextColumn,
    'dataSourceParams': iasParam,
    'onError': function(ioError) {
      ifCb(ioError, null);
    },
    'onSuccess': function(ioResult) {
      let loResult;
      if (typeof isResultKey === 'string') {
        loResult = {};
        loResult[isResultKey] = ioResult;
      } else {
        loResult = ioResult;
      }
      ifCb(null, loResult);
    }
  });
};
